function [Zmean, Xmean, statCountTime] = EPChainPostProb_back(chain, visits, theta, W1, feature, numBird, ratio, option)

%!!! since we know that in the first step, all birds are in the first
%state, so the bird distribution in the first time step is as follows. This
%is a especial case when theta = [1, 0, 0, ..., 0]

tic
EPIter = 4;

T = size(chain, 2);
dimStat = numel(theta);
global Sr;
if isempty(Sr)
    ind = (1 : (dimStat * dimStat))';
    [rowInd, ~] = ind2sub([dimStat, dimStat], ind);
    Sr = [rowInd, ind, ones(dimStat^2, 1)];
    Sr = spconvert(Sr);
end

A = zeros(dimStat, dimStat, T - 1);
for t = 1 : T - 1
    A(:, :, t) = feature2Prob(W1, feature(:, :, :, t));
end

gamma = zeros(dimStat, T);
gamma(:, 1) = theta;
stateFlag = false(dimStat, T);
stateFlag(:, 1) = gamma(:, 1) > 1e-6;
for t = 1 : T - 1
    gamma(:, t + 1) = A(:, :, t) * gamma(:, t);
    stateFlag(:, t + 1) = gamma(:, t + 1) > 1e-6;
end

% store forward pass

forwardMean = cell(T, 1);
forwardSigma = cell(T, 1);
nzflag = find(stateFlag(:, 1));
forwardMean{1} = numBird * theta(nzflag(1 : end - 1));
forwardSigma{1} = numBird * (diag(theta(nzflag(1 : end - 1))) - theta(nzflag(1 : end - 1)) * theta(nzflag(1 : end - 1))');

backwardMean = cell(T, 1);
backwardSigma = cell(T, 1);

nzflag = find(stateFlag(:, T));
curVisit = visits(nzflag, T);
obs = chain(nzflag, T);
mode = obs + 1;
mode = mode(1 : end - 1) / sum(mode) * numBird;
mode = solvePostApprox(mode, ratio, obs, curVisit, numBird);
backwardMean{T} = mode;
backwardSigma{T} = ObsHess(ratio, mode, obs, curVisit, numBird);

jointMean = zeros(2 * dimStat, T);

option.messageDirection = 'forward';

% EP iterations until converge
for t = 1 : T - 1
    nzflag1 = find(stateFlag(:, t));
    mu1 = forwardMean{t};
    Sigma1 = forwardSigma{t};
    prior1 = gamma(nzflag1(1 : end - 1), t);
    
    nzflag2 = find(stateFlag(:, t + 1));
    mu2 = backwardMean{t + 1};
    Sigma2 = backwardSigma{t + 1};
    prior2 = gamma(nzflag2(1 : end - 1), t + 1);
        
    Acur = A(nzflag2, nzflag1, t);
    curVisit = visits(nzflag1, t);
    obs = chain(nzflag1, t);
    [forMsgMean, forMsgVar, ~, ~, ~] = ...
        EPLaplaceStep(mu1, Sigma1, prior1, mu2, Sigma2, prior2, Acur, curVisit, obs, numBird, ratio, option);
    forwardMean{t + 1} = forMsgMean;
    forwardSigma{t + 1} = forMsgVar;
end

option.messageDirection = 'EP';
Xmean = zeros(dimStat, T);

for iter = 1 : EPIter
    if mod(iter, 2) == 0
        sweepIndex = 1 : T - 1;
    else
        sweepIndex = T - 1 : -1 : 1;
    end
    meanError = 0;
    varError = 0;
    for i = 1 : T - 1
        t = sweepIndex(i);
        
        nzflag1 = find(stateFlag(:, t));
        prior1 = gamma(nzflag1(1 : end - 1), t);
        mu1 = forwardMean{t};
        Sigma1 = forwardSigma{t};
        
        nzflag2 = find(stateFlag(:, t + 1));
        prior2 = gamma(nzflag2(1 : end - 1), t + 1);
        mu2 = backwardMean{t + 1};
        Sigma2 = backwardSigma{t + 1};
        
        Acur = A(nzflag2, nzflag1, t);
        curVisit = visits(nzflag1, t);
        obs = chain(nzflag1, t);
        [forMsgMean, forMsgVar, backMsgMean, backMsgVar, jmean] = ...
            EPLaplaceStep(mu1, Sigma1, prior1, mu2, Sigma2, prior2, Acur, curVisit, obs, numBird, ratio, option);
       
        if iter > 1
            meanError = meanError + sum(abs(backwardMean{t} - backMsgMean)) + sum(abs(forwardMean{t + 1} - forMsgMean));
            %varError = varError + sum(sum(abs(backwardSigma{t} - backMsgVar))) + sum(sum(abs(forwardSigma{t + 1} - forMsgVar)));
        end
        
        forwardMean{t + 1} = forMsgMean;
        forwardSigma{t + 1} = forMsgVar;
        backwardMean{t} = backMsgMean;
        backwardSigma{t} = backMsgVar;
        
        jointMean([nzflag1; dimStat + nzflag2], t) = jmean;
        Xmean(:, t) = jointMean(1 : dimStat, t);
        if t == T - 1
            Xmean(:, t + 1) = jointMean(dimStat + 1 : end, t);
        end
    end

    if iter > 1 && meanError / ((T - 1) * dimStat) < 0.1
        break;
    end
end

statCountTime = toc;
Zmean = [];
if  option.inferTransCount
    % find Z mean at each time step
    Zmean = zeros(dimStat^2, T - 1);    
    % calculate Z mean
    for t = 1 : T - 1
        
        if t == 1
            meanz = zeros(dimStat, dimStat);
            meanz(:, 1) = jointMean(dimStat + 1 : end, 1) / numBird;
            Zmean(:, t) = meanz(:);
            continue;
        end
%         
%         ZFlag = reshape(transitFlag(:, t), dimStat, dimStat);
%         [K, Q, R, missrow, misscol] = ExpandMat(ZFlag);
%         
%         ZFlag(missrow, misscol) = false;
%         flag1 = true(dimStat, 1);
%         flag2 = true(dimStat, 1);
%         flag1(misscol) = false;
%         flag2(missrow) = false;
%         
%         prob = A(:, :, t) * diag(gamma(:, t));
%         prob = prob(ZFlag(:));
%         muz = numBird * prob;
%         varz = numBird * (diag(prob) - prob * prob');
%         
%         meandiff = muz - [Q, R] * jointMean([flag1; flag2], t);
%         meanz = (K' / varz * K) \ (K' * (varz \ meandiff));
%         
%         fullmeanz = zeros(dimStat, dimStat);
%         fullmeanz(ZFlag(:)) = K * meanz + [Q, R] * jointMean([flag1; flag2], t);
%         fullmeanz(missrow, misscol) = numBird - sum(sum(fullmeanz));
%         Zmean(:, t) = fullmeanz(:);
        
        flag1 = stateFlag(:, t);
        flag2 = stateFlag(:, t + 1);
        mu1 = jointMean(1 : dimStat, t) / numBird;
        mu2 = jointMean(dimStat + 1 : dimStat + dimStat, t) / numBird;
        SubTransMean = calTransMeanFromMarginalMean(A(flag2, flag1, t) * diag(gamma(flag1, t)),  mu1(flag1), mu2(flag2), []);
        SubTransMean(SubTransMean < 0) = 0;
        SubTransMean = SubTransMean / sum(sum(SubTransMean));
        TransMean = zeros(dimStat, dimStat);
        TransMean(flag2, flag1) = SubTransMean;
        
        % return a vector with sum 1
        Zmean(:, t) = TransMean(:);
%         
%         diff = sum(sum(abs(TransMean - fullmeanz)));
%         if diff > 1
%             medvar = struct('invSigmaVec', (varz \ meandiff), 'kiv', K' * (varz \ meandiff), 'meanz', (K' / varz * K) \ (K' * (varz \ meandiff)), 'fullmeanz', fullmeanz);
%             TransMean = calTransMeanFromMarginalMean(A(:, :, t) * diag(gamma(:, t)), jointMean(1 : dimStat, t) / numBird, jointMean(dimStat + 1 : dimStat + dimStat, t) / numBird, medvar);
%             display('Error in new code');
%         end
    end
end

    function x = solvePostApprox(initx, ratio, obs, visits, numBird)
        
        opt = optimset('MaxIter', 100, 'TolX', 1e-5, 'GradObj','on', 'DerivativeCheck', 'off', 'Display', 'off');
        
        function [obj, grad] = wrapfunc(x)
            obj = ObsLik(ratio, x, obs, visits, numBird);
            grad = ObsGrad(ratio, x, obs, visits, numBird);
        end
        
        x = fminunc(@wrapfunc, initx, opt);
% 
%         %L-BFGS-B
%         l  = zeros(length(initx),1);     % no lower bound
%         u  = inf(length(initx),1);      % no upper bound
%         
%         fun     = @(x)fminunc_wrapper(x, @(x) ObsLik(ratio, x, obs, visits, numBird), @(x) ObsGrad(ratio, x, obs, visits, numBird));
%         % Request very high accuracy for this test:
%         opts    = struct();
%         opts.x0 = initx; %    The starting value (default: all zeros)
%         opts.m  = 20;                        %    Number of limited-memory vectors to use in the algorithm
%         %    Try 3 <= m <= 20. (default: 5 )
%         %       opts.factr  Tolerance setting (see this source code for more info)
%         %                       (default: 1e7 ). This is later multiplied by machine epsilon
%         opts.pgtol = 1e-5;                  %    Another tolerance setting, relating to norm(gradient,Inf)
%         %    (default: 1e-5)
%         opts.maxits = 500;                  %    How many iterations to allow (default: 100)
%         %       opts.maxTotalIts    How many iterations to allow, including linesearch iterations
%         %                       (default: 5000)
%         opts.printEvery = 10000000;         %    opts.printEvery     How often to display information (default: 1)
%         % Run the algorithm:
%         [x, ~, info] = lbfgsb(fun, l, u, opts);
    end



end

%% old code for calculate Zmean
%         ZFlag = reshape(transitFlag(:, t), dimStat, dimStat);
%         [K, Q, R, missrow, misscol] = ExpandMat(ZFlag);
%         
%         ZFlag(missrow, misscol) = false;
%         flag1 = true(dimStat, 1);
%         flag2 = true(dimStat, 1);
%         flag1(misscol) = false;
%         flag2(missrow) = false;
%         
%         prob = A(:, :, t) * diag(gamma(:, t));
%         prob = prob(ZFlag(:));
%         muz = numBird * prob;
%         varz = numBird * (diag(prob) - prob * prob');
%         
%         meandiff = muz - [Q, R] * jointMean([flag1; flag2], t);
%         meanz = inv(K' * inv(varz) * K) * K' * (varz \ meandiff);
%         
%         fullmeanz = zeros(dimStat, dimStat);
%         fullmeanz(ZFlag(:)) = K * meanz + [Q, R] * jointMean([flag1; flag2], t);
%         fullmeanz(missrow, misscol) = numBird - sum(sum(fullmeanz));
%         Zmean(:, t) = fullmeanz(:);