function MeanZ = calTransMeanFromMarginalMean(ProbZ, mux, muy, medvar)

% the relation between transition counts and marginal sum
% Z = K Z~ + P x + Q y

% the conditional distribution of Z' given x and y is 
% mu = (K' SigmaZ^-1 K)^-1 K' SigmaZ^-1 (muZ - P x - Q y)
% Sigma = (K' SigmaZ^-1 K)^-1

% the mean of Z~ is 
% mu = (K' SigmaZ^-1 K)^-1 K' SigmaZ^-1 (muZ - P mu_x - Q mu_y)

% the mean of Z is 
% mu = (K' SigmaZ^-1 K)^-1 K' SigmaZ^-1 (muZ - P mu_x - Q mu_y) + P mu_x + Q mu_y

% muZ = N probZ
% SigmaZ = N(diag(probZ) = probZ * probZ') 

% Suppose the K matrix is [I; B]
% and B is like 
% [1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0; 
%  0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0;
%  0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1;
%  1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0; 
%  0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0;
%  0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0;
%  0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1]

% matrix P is like
% [ 0
%   I
%   0]

% matrix Q is like
% [ 0
%   0
%   I]

% ProbZ = rand(5, 7);
% ProbZ = ProbZ / sum(ProbZ(:));
% mux = rand(size(ProbZ, 2), 1);
% mux = mux / sum(mux);
% muy = rand(size(ProbZ, 1), 1);
% muy = muy / sum(muy);


[dim2, dim1] = size(ProbZ);

% First calculate B
Btrans = - [kron(eye(dim1 - 1), ones(dim2 - 1, 1)), kron(ones(dim1 - 1, 1), eye(dim2 - 1))];



% move a nonzero elment to the bottom right corner by switching rows and columns
[~, maxInd] = max(ProbZ(:));
[compInd2, compInd1] = ind2sub([dim2, dim1], maxInd(1));

flag2 = true(dim2, 1);
flag2(compInd2) = false;
flag1 = true(dim1, 1);
flag1(compInd1) = false;

% adding a small number to avoid singlar matrix
bottomProbZ = ProbZ(compInd2, flag1)' + 1e-8;  
rightProbZ = ProbZ(flag2, compInd1) + 1e-8;

resProbZ = ProbZ(compInd2, compInd1);
prob1 = ProbZ(flag2, flag1);
prob1 = prob1(:);
prob2 = [bottomProbZ; rightProbZ];

% adding the number too, since mu1 is the sum of all rows and mu2 is the
% sum of all columns
mu1 = mux(flag1) + 1e-8;
mu2 = muy(flag2) + 1e-8;

%assert variables for assertion
% probz = [prob1; prob2];
% K = [eye(numTop); Btrans'];
%assert

% calculating the part vec1 =  SigmaZ^{-1}(mu_Z - P mu_x - Q mu_y) 
% = (1 - sum mu_x - sum mu_y) 1 / resProbZ - [0; mup1; 0] - [0; 0; mup2]
scale1 = (1 - sum(mu1) - sum(mu2)) / resProbZ;
% % assert  
%mup1 = mu1 ./ bottomProbZ; 
%mup2 = mu2 ./ rightProbZ; 
%lowerv = ones(dim1 + dim2 - 2, 1) * scale1 - [mup1; mup2];
% SigmaZ = diag(probz) - probz * probz';
% vec1 = SigmaZ \ (probz - [zeros(numTop, 1); mu1; mu2]);
% diff = sum(abs(vec1 - [scale1 * ones(numTop, 1); lowerv]) ./ abs(vec1));
% diff < 1e-8
% % assert

% % assert the inverse form of (K' * SigmaZ^-1 * K)^-1
% P1 = diag(probz(1 : numTop));
% P2 = diag(probz(numTop + 1 : end));
% H = Btrans' * P1 * Btrans;
% Temp1 = P1 * Btrans * inv(P2 + H) * Btrans' * P1;
% Temp2 = P1 - Temp1;
% Mat2 = Temp2 - Temp2 * ones(numTop) * Temp2 / (1 - sum(probz(numTop + 1 : end)) - sum(sum(Temp1))) ;
% Mat0 = inv(K' * inv(diag(probz) - probz * probz') * K);
% diff = sum(sum(abs(Mat2 - Mat0) ./ abs(Mat0)));
% %assert

% first calculate H = B * P1 * B'
ProbSubZ = reshape(prob1, dim2 - 1, dim1 - 1);
subColSum = sum(ProbSubZ, 1)';
subRowSum = sum(ProbSubZ, 2);
H = [diag(subColSum), ProbSubZ'; ProbSubZ, diag(subRowSum)];

% then calculate Temp2 * 1
invvec1 = prob1 .* (Btrans * ((diag(prob2) + H) \ [- subColSum; - subRowSum]));
invvec2 = prob1 - invvec1;
dominator = 1 - sum(prob2) - sum(invvec1);
multiplyInv1 = invvec2 - invvec2 * (sum(invvec2) / dominator);

% %assert multiplyInv1 == inv(K' * inv(diag(probz) - probz * probz') * K) * ones(numTop, 1)
% vec1 = inv(K' * inv(diag(probz) - probz * probz') * K) * ones(numTop, 1);
% diff = sum(abs(multiplyInv1 - vec1) ./ abs(vec1));
% % assert

% then calculate Temp2 * B' * lowerv
% bvec1 = prob1 .* (Btrans * ((diag(prob2) + H) \ (H * lowerv)));
% bvec2 = prob1 .* (Btrans * lowerv) - bvec1;
% multiplyInvB1 = bvec2 - invvec2 * (sum(bvec2) / dominator);

% dplowerv = diag(prob2) * lowerv
dplowerv = prob2 * scale1 - [mu1; mu2];
bvec2 = prob1 .* (Btrans * ((diag(prob2) + H) \ dplowerv));
multiplyInvB1 = bvec2 - invvec2 * (sum(bvec2) / dominator);

% %assert
% vec1 = inv(K' * inv(diag(probz) - probz * probz') * K) * Btrans * lowerv;
% diff = sum(abs(multiplyInvB1 - vec1) ./ abs(vec1));
% % assert

% then we can get the mean of top left z 
musubz = scale1 * multiplyInv1 + multiplyInvB1;
% %assert
% vec1 = inv(K' * inv(diag(probz) - probz * probz') * K) * K' * inv(diag(probz) - probz * probz') * (probz - [zeros(numTop, 1); mu1; mu2]);
% diff = sum(abs(musubz - vec1) ./ abs(vec1));
% % assert

SubzM = reshape(musubz, dim2 - 1, dim1 - 1);
bottomMu = - sum(SubzM, 1)' + mu1;
rightMu = - sum(SubzM, 2) + mu2;

% make it a matrix and recover the row and column order
MeanZ = zeros(dim2, dim1);
MeanZ(flag2, flag1) = SubzM;
MeanZ(flag2, compInd1) = rightMu;
MeanZ(compInd2, flag1) = bottomMu';
MeanZ(compInd2, compInd1) = 1 - sum(musubz) - sum(rightMu) - sum(bottomMu);







