function [progress, time] = em_learning(data, randStart, option)

% This is a correct version of LGFilter. 
% On the main version, some update is to be applied on the M step part. 

emIter = option.numEM;
optIter = 500;

chainData = data.chainData;
visits = data.visits;
theta = data.theta; 
feature = data.feature; 
numBird = data.numBird;
ratio = data.ratio;

numChain = length(chainData);
dimObs = size(chainData{1}, 1);
dimStat = dimObs;
dimFeature = size(feature{1}, 1);
% assuming all chains has the same length
T = size(chainData{1}, 2);
trueW1 = data.trueW1;

global Sr Sc;
ind = (1 : (dimStat * dimStat))';
[rowInd, colInd] = ind2sub([dimStat, dimStat], ind);
Sr = [rowInd, ind, ones(dimStat^2, 1)];
Sr = full(spconvert(Sr));
Sc = [colInd, ind, ones(dimStat^2, 1)];
Sc = full(spconvert(Sc));


W1 = randStart;

progress = zeros(1, emIter);
time = zeros(1, emIter);

for iter = 1 : emIter
   
    tic
    display(['Iteration ', num2str(iter)]);
    oldW1 = W1;    
    
    % E step
    Zmean = cell(length(chainData), 1);
    for ich = 1 : numChain
        Zmean{ich} = zeros(dimStat^2, T - 1);
    end
    
    for ich = 1 : numChain
        Zmean{ich} = EPChainPostProb(chainData{ich}, visits{ich}, theta, W1, feature{ich}, numBird, ratio, option);  
    end

    % M step
    % optimization of theta, which is the mean of all posterior means
    for iopt = 1 : optIter
        
        H = zeros(dimFeature, dimFeature);
        grad = zeros(dimFeature, 1);
        obj = 0;
        for ich = 1 : numChain
            for t = 1 : T - 1
                A = feature2Prob(W1, feature{ich}(:, :, :, t));
                Z = reshape(Zmean{ich}(:, t), dimStat, dimStat);            
                obj = obj + sum(sum(Z .* log(A)));
                for icell = 1 : dimStat
                    tempFeat = squeeze(feature{ich}(:, icell, :, t));
                    grad = grad + tempFeat * Z(:, icell) - sum(Z(:, icell)) * tempFeat * A(:, icell);
                    meanFeat = tempFeat * A(:, icell);
                    H = H + sum(Z(:, icell)) * (meanFeat * meanFeat' - tempFeat * diag(A(:, icell)) * tempFeat');
                    if t == 1
                        break
                    end
                end
                
            end
        end
        step = - H \ grad;
        if  max(abs(step)) > 1
            step = step / max(abs(step));
        end
        W1 = W1 + step;
        if sum(abs(step)) < 1e-3
            break;
        end
    end
    
    fitError = norm(W1 - trueW1, 1) / norm(trueW1, 1)
    progress(iter) = fitError;
    time(iter) = toc;
    
%     if mean(abs(oldW1 - W1)) < 1e-4
%         break;
%     end
    
end

progress = progress(1 : iter);
time = time(1 : iter);

