function learnParameter()
  
%%

s = RandStream('mcg16807', 'Seed', 1);
RandStream.setGlobalStream(s);

% chain data is a synthesized bird migration problem. 
% It contains data collected from three Markov chains. The following is the
% detailed explanation of the fields of ``data'':
% theta:      the initial distribution of birds
% chainData:  noised oberservation/counts of birds in three Markov chains.
%             Only visited map cells have observations. But in this copy of
%             data, all map cells are visited. 
% feature:    synthetic weather feature affecting transition probabilities.
%             For each transition there is a set of feature f_ijt, whose
%             inner product with trueW1 is the potential of transition from
%             i to j at time t.
% visits:     the flag indicating whether a cell is visited. All flags are
%             1 in this dataset.
% 
% numBird:    number of birds in this transition problem
% ratio:      the paramter in the observation process. 
%             y_i = Poisson(x_i * ratio), where x_i is the number of bird 
%             actually in the map cell and y_i is the number of birds
%             observed.
% 
% trueW1:     the true parameter w governing the transition probabilities
%             in the migration model. This parameter is to be learned. 



data = load('chain_data');


numEM = 50;   
randStart = rand(size(data.trueW1));

option = struct('SigmaType', 'prior', 'inferTransCount', true, 'numEM', numEM);
[progress, time] = em_learning(data, randStart, option);
time = cumsum(time);

